#include "BeatDetector.h"

#include "MusicCommons.h"

#include <stdio.h>

using namespace std;

BeatDetector::BeatDetector( uint32_t _history_depth ) :
	history_depth_(_history_depth)
{
	history_ = new deque<RT_SAMPLE>();
	//complex_history_ = new deque<complex>();

	//complex c;
	//c.re = 0; c.im = 0;
	for( uint32_t i = 0; i < history_depth_; i++ ) {
		history_->push_back( 0 );
		//complex_history_->push_back( c );
	}

	history_sum_ = 0;
	//complex_history_sum_.re = 0;
	//complex_history_sum_.im = 0;
	beat_ = false;
	old_beat_ = false;
	print_ = false;
	analyzed_ = 0;
}

BeatDetector::~BeatDetector()
{
	delete history_;
}

void BeatDetector::tick( RT_SAMPLE _instant_average )
{
	++analyzed_;

	RT_SAMPLE local_average = history_sum_ / history_->size();
	RT_SAMPLE variance = variance_from( history_, local_average );
	RT_SAMPLE ratio = _instant_average / local_average;
	// -84 is a bit of 'secret sauce' - the given equation used a tiny number,
	//  something like -.0025... which seemed better tuned for integer valued
	//  amplitudes. Since we're scaling integer values by 32768, it seemed
	//  appropriate to scale the given value by the same.
	const float threshold_base = 1.5142857f;
	const float variance_influence = -84.0f;
	RT_SAMPLE threshold = (variance_influence * variance) + threshold_base;

	if( print_ ) {
		fprintf( stderr, "%f %f : %.3f / %.3f (%.7f)",
				local_average, _instant_average, ratio, threshold,
				variance*variance_influence );
	}

	//beat_ = (ratio >= threshold);
	// Instead of straight ratio, set up a sort of Schmitt Trigger
	// Track beat for this sample and last sample, and only return is_beat
	// on rising edge
	old_beat_ = beat_;
	if( ratio >= threshold ) {
		beat_ = true;
	} else if( _instant_average < local_average ) {
		beat_ = false;
	}

	if( print_ ) {
		if( beat_ ) {
			fprintf( stderr, " => BEAT!" );
		}
		fprintf( stderr, "\n" );
	}

	history_sum_ += _instant_average - history_->front();

	history_->push_back( _instant_average );
	history_->pop_front();
}

/*
void BeatDetector::tick( complex _instant_average )
{
	complex_history_sum_.re +=
		fabs( _instant_average.re ) - fabs( complex_history_->front().re );
	complex_history_sum_.im +=
		fabs( _instant_average.im ) - fabs( complex_history_->front().im );

	complex_history_->push_back( _instant_average );
	complex_history_->pop_front();
}
*/
