#include "MusicCommons.h"

#include <math.h>
#include <assert.h>

using namespace std;

uint32_t g_subband_width[NSUBBANDS];

RT_SAMPLE variance_from( deque<RT_SAMPLE> *buffer, RT_SAMPLE avg )
{
	uint32_t n_samples = buffer->size();

	RT_SAMPLE variance = 0;
	for( deque<RT_SAMPLE>::iterator iter = buffer->begin();
			iter != buffer->end(); iter++ ) {
		variance += pow( *iter - avg, 2 );
	}
	variance /= n_samples;

	return variance;
}

// Can precompute this
// wi = a + i * b
//	For 64 bands with 1024 samples (but we really have 512?) a = 2, b = 0.46
//	For 64 bands with 512 samples a = 2, b = 0.206
//	For 32 bands with 512 samples a = 2, b = 0.935
//	For 32 bands with 512 samples a = 3, b = 0.87
//	For 32 bands with 512 samples a = 4, b = 0.8
// wi = a + b ^ i * w0, w0 = 1
//  For 32 bands with 512 samples a = 0, b = 1.1396
//  For 32 bands with 512 samples a = 2, b = 1.133907
//  For 32 bands with 512 samples a = 2, b = 0.978
uint32_t subband_width( uint32_t sb )
{
	if( NSUBBANDS == 64 ) {
		return 2 + 0.206 * sb;
	} else if( NSUBBANDS == 32 ) {
		//return 2 + pow( sb, 0.978 );
		return 2 + 0.935 * sb;
	} else {
		assert( false && "Unsupported NSUBBANDS" );
	}
}

void init_global_subband_widths()
{
	for( uint32_t sb = 0; sb < NSUBBANDS; sb++ ) {
		g_subband_width[sb] = subband_width( sb );
	}
}
