#include "Fault.h"

#include <stdlib.h>
#include <time.h>

#include <iostream>

using namespace std;

Fault::Fault( Ogre::Terrain *_terrain ) : TerrainEvent( _terrain )
{
	points_ = size_ * size_;
	displacement_ = 0;
}

// Mostly we won't be creating random events, but size will be involved
Fault *Fault::Random( Ogre::Terrain *_terrain )
{
	Fault *te = new Fault( _terrain );

	// Note: event parameters are terrain-independent
	float start, end;
	start = (float)rand() / (float)RAND_MAX;
	end = (float)rand() / (float)RAND_MAX;

	te->pointsIs( start, end );
	te->displacementIs( rand() % 60 - 30 );

	return te;
}

Fault::~Fault()
{
}

// Convert points to Ax + By + C = 0 (normal) form
void Fault::pointsIs(
	long _x_start, long _y_start,
	long _x_end, long _y_end )
{
	// Avoid explicitly handling same point; these will result in all points
	// being mapped to zero in the check. This degenerate line is handled in
	// Morph.
	a_ = _y_end - _y_start;
	b_ = _x_start - _x_end;
	c_ = _x_end * _y_start - _x_start * _y_end;
}

void Fault::pointsIs( float _pct_start, float _pct_end )
{
	unsigned long start = _pct_start * points_;
	unsigned long end = _pct_end * points_;

	long x_start = start % size_;
	long y_start = start / size_;

	long x_end = end % size_;
	long y_end = end / size_;

	pointsIs( x_start, y_start, x_end, y_end );
}

// TODO Why they heck would the direct manipulation of height data work in
// Smooth::Morph, but not in Fault::Morph?
void Fault::Morph()
{
	// No-op if it's a non-defined line
	if( a_ == b_ && a_ == c_ && a_ == 0 ) {
		return;
	}

	if( displacement_ == 0 || gain_ == 0 ) {
		return;
	}

	displacement_ *= gain_;

	for( long x = 0; x < size_; x++ ) {
		for( long y = 0; y < size_; y++ ) {
			bool aboveLine = ( a_ * x + b_ * y + c_ >= 0 );

			if( aboveLine ) {
				terrain_->setHeightAtPoint( x, y,
						terrain_->getHeightAtPoint( x, y ) + displacement_ );
			} else {
				terrain_->setHeightAtPoint( x, y,
						terrain_->getHeightAtPoint( x, y ) - displacement_ );
			}
		}
	}
}

void Fault::Morph( float *_height_data )
{
	if( a_ == b_ && a_ == c_ && a_ == 0 ) {
		return;
	}

	if( displacement_ == 0 || gain_ == 0 ) {
		return;
	}

	// Compute a_ * x + b_ * y + c_ 
	// Subtracting a_ and b_ accounts for zero index
	long test_stat = c_ - a_ - b_;

	for( long x = 0; x < size_; x++ ) {
		test_stat += a_;
		long reset = 0;
		int loc = x - size_;

		for( long y = 0; y < size_; y++ ) {
			test_stat += b_;
			reset += b_;

			loc += size_;

			bool aboveLine = test_stat >= 0;

			if( aboveLine ) {
				_height_data[ loc ] += displacement_;
			} else {
				_height_data[ loc ] -= displacement_;
			}
		}
		test_stat -= reset;
	}
}
