#include "Smooth.h"

#include <iostream>

Smooth::Smooth( Ogre::Terrain *_terrain ) : TerrainEvent( _terrain )
{
	force_ = 1.0f;
	size_ = _terrain->getSize();
}

Smooth *Smooth::Random( Ogre::Terrain *_terrain )
{
	Smooth *te = new Smooth( _terrain );
	return te;
}

Smooth::~Smooth()
{
}

// Concept based on http://www.nfostergames.com/Lessons/TerrainSmoothing.htm
// The actual code is entirely my own
void Smooth::Morph()
{
	if( force_ <= 0 || gain_ <= 0 ) {
		return;
	}

	force_ = gain_;

	float scratch_height[size_][size_];

	float *height_data = terrain_->getHeightData();

	for( int x = 0; x < size_; x++ ) {
		for( int y = 0; y < size_; y++ ) {
			scratch_height[x][y] = AverageNeighborHeight( height_data, x, y );
		}
	}

	for( int x = 0; x < size_; x++ ) {
		for( int y = 0; y < size_; y++ ) {
			height_data[x + y * size_] = 
				force_ * scratch_height[x][y] +
				( 1.0 - force_ ) * height_data[x + y * size_];
		}
	}
}

void Smooth::Morph( float *_height_data )
{
	float scratch_height[size_][size_];

	if( force_ <= 0 || gain_ <= 0 ) {
		return;
	}

	force_ = gain_; // Whatever

	for( int x = 0; x < size_; x++ ) {
		for( int y = 0; y < size_; y++ ) {
			scratch_height[x][y] = AverageNeighborHeight( _height_data, x, y );
		}
	}

	for( int x = 0; x < size_; x++ ) {
		for( int y = 0; y < size_; y++ ) {
			_height_data[x + y * size_] =
				force_ * scratch_height[x][y] +
				( 1.0 - force_ ) * _height_data[x + y * size_];
		}
	}
}

float Smooth::AverageNeighborHeight( float *_height_data, int _x, int _y )
{
	int neighbors = 0;
	float patch_height = 0.0f;

	for( int xn = _x-1; xn <= _x+1; xn++ ) {
		// Boundary conditions
		if( xn < 0 || xn >= size_ ) {
			continue;
		}

		for( int yn = _y-1; yn <= _y+1; yn++ ) {
			// Boundary conditions
			if( yn < 0 || yn >= size_ ) {
				continue;
			}

			// Do not consider the square itself
			if( yn == 0 && xn == 0 ) {
				continue;
			}

			neighbors++;
			patch_height += _height_data[ xn + yn * size_ ];
		}
	}

	return patch_height / neighbors;
}
