#include "TerrainEvent.h"
#include "Fault.h"
#include "Smooth.h"
#include "Blend.h"
#include "Song.h"

#include <cassert>

/*
 * TODO: TerrainEvent shouldn't create events from ProtoEvents, it should just
 *  hold its ProtoEvent until called for, then translate on-demand
 *  It could even potentially inherit from ProtoEvent, which actually makes a
 *  lot of sense.
 *  As it is, I'm doing a lot of copying and code duplication.
 *  For now, we will write this off as 'commit whatever sins you must in order
 *  to get it working, then remove duplication until it looks right'
 */

TerrainEvent::TerrainEvent( Ogre::Terrain *_terrain ) : terrain_(_terrain)
{
	size_ = terrain_->getSize();
	gain_ = 1.0f;
}

TerrainEvent::~TerrainEvent()
{
}

Fault *TerrainEvent::MakeFault( Ogre::Terrain *_terrain )
{
	return new Fault( _terrain );
}

Fault *TerrainEvent::RandomFault( Ogre::Terrain *_terrain )
{
	return Fault::Random( _terrain );
}

Smooth *TerrainEvent::MakeSmooth( Ogre::Terrain *_terrain )
{
	return new Smooth( _terrain );
}

Smooth *TerrainEvent::RandomSmooth( Ogre::Terrain *_terrain )
{
	return Smooth::Random( _terrain );
}

Blend *TerrainEvent::MakeBlend( Ogre::Terrain *_terrain )
{
	return new Blend( _terrain );
}

Blend *TerrainEvent::RandomBlend( Ogre::Terrain *_terrain )
{
	return Blend::Random( _terrain );
}

Song *TerrainEvent::MakeSong( Ogre::Terrain *_terrain )
{
	return new Song( _terrain );
}

Song *TerrainEvent::RandomSong( Ogre::Terrain *_terrain )
{
	return Song::Random( _terrain );
}

// TODO Hacky. Should use double-dispatch pattern.
TerrainEvent *TerrainEvent::FromProto(
		Ogre::Terrain *_terrain, ProtoEvent *_event )
{
	ProtoFault *proto_fault;
	ProtoSmooth *proto_smooth;
	ProtoBlend *proto_blend;
	ProtoSong *proto_song;

	TerrainEvent *newEvent = NULL;

	if( ( proto_fault = dynamic_cast<ProtoFault *>( _event ) ) ) {
		newEvent = FromProto( _terrain, proto_fault );
	} else if( ( proto_smooth = dynamic_cast<ProtoSmooth *>( _event ) ) ) {
		newEvent = FromProto( _terrain, proto_smooth );
	} else if( ( proto_blend = dynamic_cast<ProtoBlend *>( _event ) ) ) {
		newEvent = FromProto( _terrain, proto_blend );
	} else if( ( proto_song = dynamic_cast<ProtoSong *>( _event ) ) ) {
		newEvent = FromProto( _terrain, proto_song );
	}

	assert( newEvent && "Unexpected ProtoEvent type" );

	newEvent->indexIs( _event->index() );

	return newEvent;
}

Fault *TerrainEvent::FromProto( Ogre::Terrain *_terrain, ProtoFault *_proto )
{
	Fault *newFault = TerrainEvent::MakeFault( _terrain );

	newFault->startTimeIs( _proto->startTime() );
	newFault->endTimeIs( _proto->endTime() );
	newFault->displacementIs( _proto->displacement() );
	newFault->pointsIs( _proto->startPos(), _proto->endPos() );

	return newFault;
}

Smooth *TerrainEvent::FromProto( Ogre::Terrain *_terrain, ProtoSmooth *_proto )
{
	Smooth *newSmooth = TerrainEvent::MakeSmooth( _terrain );

	newSmooth->startTimeIs( _proto->startTime() );
	newSmooth->endTimeIs( _proto->endTime() );
	newSmooth->forceIs( _proto->force() );

	return newSmooth;
}

Blend *TerrainEvent::FromProto(	Ogre::Terrain *_terrain, ProtoBlend *_proto )
{
	Blend *newBlend = TerrainEvent::MakeBlend( _terrain );

	return newBlend;
}

Song *TerrainEvent::FromProto( Ogre::Terrain *_terrain, ProtoSong *_song )
{
	Song *newSong = TerrainEvent::MakeSong( _terrain );

	newSong->startTimeIs( _song->startTime() );
	newSong->endTimeIs( _song->endTime() );
	newSong->filenameIs( _song->filename() );

	return newSong;
}
