#ifndef __TERRAIN_EVENT_H__
#define __TERRAIN_EVENT_H__

#include <Terrain/OgreTerrain.h>

#include "TimedEvent.h"
#include "ProtoFault.h"
#include "ProtoSmooth.h"
#include "ProtoBlend.h"
#include "ProtoSong.h"

class Fault;
class Smooth;
class Blend;
class Song;

class TerrainEvent : public TimedEvent
{
	/* Public TerrainEvent interface */

	public:
		virtual ~TerrainEvent();

		virtual void Morph() = 0;
		virtual void Morph( float *height_data ) = 0;

		float gain_;

		inline void indexIs( long _index ) { index_ = _index; }
		inline long index() { return index_; }

	protected:
		TerrainEvent( Ogre::Terrain *_terrain );

		Ogre::Terrain *terrain_;

		int size_;
		long index_;

	/* Constructional Interface */
	/* TODO This should be refactored to a separate class */

	public:
		static Fault *MakeFault( Ogre::Terrain *_terrain );
		static Smooth *MakeSmooth( Ogre::Terrain *_terrain );
		static Blend *MakeBlend( Ogre::Terrain *_terrain );
		static Song *MakeSong( Ogre::Terrain *_terrain );

		static Fault *RandomFault( Ogre::Terrain *_terrain );
		static Smooth *RandomSmooth( Ogre::Terrain *_terrain );
		static Blend *RandomBlend( Ogre::Terrain *_terrain );
		static Song *RandomSong( Ogre::Terrain *_terrain );

		static TerrainEvent *FromProto(
				Ogre::Terrain *_terrain, ProtoEvent *_event );

	private:
		static Fault *FromProto(
				Ogre::Terrain *_terrain, ProtoFault *_proto );
		static Smooth *FromProto(
				Ogre::Terrain *_terrain, ProtoSmooth *_proto );
		static Blend *FromProto(
				Ogre::Terrain *_terrain, ProtoBlend *_proto );
		static Song *FromProto(
				Ogre::Terrain *_terrain, ProtoSong *_song );
};

#endif
