#ifndef __VIEWER_H__
#define __VIEWER_H__

#include "BaseApplication.h"

#include "TerrainEvent.h"
#include "Script.h"
#include "Fault.h"
#include "Smooth.h"
#include "Blend.h"

#include "StereoPCM.h"

#include <Terrain/OgreTerrain.h>
#include <Terrain/OgreTerrainGroup.h>

#include <queue>
#include <vector>
#include <set>

#include "RtAudio.h"
#include "RtError.h"

typedef std::priority_queue< TerrainEvent*, \
					std::vector<TerrainEvent*>, \
					OrderTimedEvent > \
					TerrainEventQueue;

typedef std::set< TerrainEvent* > TerrainEventSet;

class Viewer : public BaseApplication
{
	private:
		Ogre::TerrainGlobalOptions *mTerrainGlobals;
		Ogre::TerrainGroup* mTerrainGroup;
		bool mTerrainsImported;
		OgreBites::Label* mInfoLabel;

		Ogre::Timer* mRenderTime;

		RtAudio *mAudio;

		void defineTerrain(long x, long y);
		void initBlendMaps(Ogre::Terrain* terrain);
		void configureTerrainDefaults(Ogre::Light* light);
		TerrainEventQueue *pending_terrain_events_;
		TerrainEventSet *active_terrain_events_;

		Script *script_;

	public:
		Viewer(void);
		virtual ~Viewer(void);

		unsigned long mSimTimeMS;

		StereoPCM playlist_[N_SONGS];

		inline void audioIs( RtAudio *audio ) {
			mAudio = audio;
		}

		inline RtAudio *audio() {
			return mAudio;
		}

		Ogre::Vector3 getUserTerrainPos();
	protected:
		virtual void createScene(void);
		virtual void createFrameListener(void);
		virtual void destroyScene(void);
		virtual bool frameStarted(const Ogre::FrameEvent& evt);
		virtual bool frameRenderingQueued(const Ogre::FrameEvent& evt);

		virtual bool mousePressed( const OIS::MouseEvent &arg, OIS::MouseButtonID id );
};

#endif
