#include "Viewer.h"

#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
#define WIN32_LEAN_AND_MEAN
#include "windows.h"
#endif

#include "RtAudio.h"
#include "Mad.h"
#include "Units.h"

#include <stdint.h>
#include <fcntl.h>

using namespace std;

#ifdef __cplusplus
extern "C" {
#endif

uint32_t g_buffer_size;

static int audioCallback( void *outputBuffer, void *inputBuffer,
		unsigned int nFrames, double streamTime,
		RtAudioStreamStatus status, void *userData )
{
	(void)inputBuffer;
	(void)streamTime;
	(void)status;

	RT_SAMPLE *out = (RT_SAMPLE *)outputBuffer;
	Viewer *app = (Viewer *)userData;

	for( unsigned int i = 0; i < 2 * nFrames; i += 2 ) {
		unsigned int left = i;
		unsigned int right = i + 1;
		out[left] = 0;
		out[right] = 0;
	}

	for( int i = 0; i < N_SONGS; i++ ) {
		Ogre::Vector3 userAt = app->getUserTerrainPos();
		StereoPCM *song = &app->playlist_[i];
		int xdist = userAt.x - song->center_x;
		int ydist = userAt.y - song->center_y;
		float distance = sqrt( xdist * xdist + ydist * ydist );
		float gain =
			( song->influence - distance ) / song->influence;

		if( gain > 0 ) {
			gain = gain * gain;
			for( unsigned int j = 0; j < 2 * nFrames; j += 2 ) {
				unsigned int left = j;
				unsigned int right = j + 1;

				assert( song->left->size() == song->right->size() );

				if( song->left->size() ) {
					out[left] += gain * song->left->front() / 32768.0f;
					out[right] += gain * song->right->front() / 32768.0f;
					song->left->pop_front();
					song->right->pop_front();
				}
			}
		} else {
			for( unsigned int j = 0; j < 2 * nFrames; j += 2 ) {
				if( song->left->size() ) {
					song->left->pop_front();
					song->right->pop_front();
				}
			}
		}
	}

	app->mSimTimeMS += ( (float)nFrames / 44100.0f ) * 1000.0f;

	return 0;
}

#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
	INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR strCmdLine, INT )
#else
		int main(int argc, char *argv[])
#endif
		{
			// Create audio object
			RtAudio *audio = NULL;
			RtAudio::StreamParameters outParams;

			g_buffer_size = 1024;

			// Create application object
			Viewer app;

			try {
				audio = new RtAudio();

				outParams.deviceId = audio->getDefaultOutputDevice();
				outParams.nChannels = 2;

				audio->openStream(
						&outParams,
						NULL,
						RTAUDIO_FLOAT32,
						44100,
						&g_buffer_size,
						audioCallback,
						&app );
			} catch( RtError & err ) {
				cout << err.getMessage();
				if( audio ) {
					delete audio;
				}
				exit( 1 );
			}

			app.audioIs( audio );

			try {
				app.go();
			} catch( Ogre::Exception& e ) {
#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
				MessageBox( NULL, e.getFullDescription().c_str(), "An exception has occured!", MB_OK | MB_ICONERROR | MB_TASKMODAL);
#else
				std::cerr << "An exception has occured: " <<
					e.getFullDescription().c_str() << std::endl;
#endif
			}

			return 0;
		}

#ifdef __cplusplus
}
#endif

